from datetime import datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, Field, model_validator
from pydantic.alias_generators import to_camel


class ProductInfo(BaseModel):
    model_config = ConfigDict(
        extra="allow",
        alias_generator=to_camel,
        populate_by_name=True,
    )


class ProductsConfig(ProductInfo):
    values: dict


class BondsRequest(ProductInfo):
    bond_issuer_type_id: int
    bond_exchange_id: int
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class ETFsRequest(ProductInfo):
    popular_only: bool
    input_aggregate_types: str
    input_aggregate_values: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class FundsRequest(ProductInfo):
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class FuturesRequest(ProductInfo):
    future_exchange_id: int
    underlying_isin: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class LeveragedsRequest(ProductInfo):
    popular_only: bool
    input_aggregate_types: str
    input_aggregate_values: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str
    underlying_product_id: int | None = Field(default=None)
    shortlong: str | None = Field(default=None)


class LookupRequest(ProductInfo):
    search_text: str
    limit: int = Field(default=5)
    offset: int = Field(default=0)
    product_type_id: int


class OptionsRequest(ProductInfo):
    input_aggregate_types: str
    input_aggregate_values: str
    option_exchange_id: int
    underlying_isin: str
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class StocksRequest(ProductInfo):
    exchange_id: int | None = Field(default=None)
    is_in_us_green_list: bool | None = Field(default=None, alias="isInUSGreenList")
    index_id: int | None = Field(default=None)
    offset: int = Field(default=10)
    limit: int = Field(default=0)
    require_total: bool = Field(default=False)
    search_text: str | None = Field(default=None)
    sort_columns: str = Field(default="name")
    sort_types: str = Field(default="asc")
    stock_country_id: int | None = Field(default=None)


class WarrantsRequest(ProductInfo):
    search_text: str
    offset: int
    limit: int
    require_total: bool
    sort_columns: str
    sort_types: str


class ProductBatch(BaseModel):
    offset: int
    products: list[dict] | None = Field(default=None)
    response_datetime: datetime = Field(default_factory=datetime.now)
    total: int = Field(default=0)


class UnderlyingsRequest(ProductInfo):
    future_exchange_id: int | None = Field(default=None)
    option_exchange_id: int | None = Field(default=None)

    int_account: int | None = Field(default=None)
    session_id: str | None = Field(default=None)

    @model_validator(mode="before")
    @classmethod
    def one_of(cls, data: Any):
        if isinstance(data, dict):
            if "future_exchange_id" in data and "option_exchange_id" in data:
                raise ValueError(
                    "Can't set both: `future_exchange_id` and `option_exchange_id`."
                )
            if "future_exchange_id" not in data and "option_exchange_id" not in data:
                raise ValueError(
                    "One of these parameters should be set: `future_exchange_id` or `option_exchange_id`."
                )

        return data


class Underlying(ProductInfo):
    contract_type: int | None = Field(default=None)
    isin: str | None = Field(default=None)
    name: str | None = Field(default=None)
    symbol: str | None = Field(default=None)
    underlying_name: str | None = Field(default=None)
    underlying_product_id: int | None = Field(default=None)



import os
from pathlib import Path

from orjson import loads
from pydantic import BaseModel, Field, validator


class Credentials(BaseModel):
    int_account: int | None = Field(default=None)
    username: str
    password: str
    totp_secret_key: str | None = Field(default=None)
    one_time_password: int | None = Field(default=None)

    @validator("totp_secret_key")
    @classmethod
    def one_of(cls, v, values, **kwargs):
        _ = kwargs
        if "one_time_password" in values and values["one_time_password"] is not None:
            raise ValueError(
                "You can't set both `one_time_password` and `totp_secret_key`."
            )
        return v


def build_credentials(location: str = "", override: dict | None = None) -> Credentials:
    if not location and not override:
        raise AttributeError("You need to provide at least on argument.")

    if location:
        location_path = Path(location)
    else:
        location_path = None

    env_content = os.environ.get("DEGIRO_ACCOUNT")

    if env_content:
        config = loads(env_content)
    elif location_path and location_path.exists():
        config = loads(location_path.read_text())
    else:
        config = {}

    if override:
        config.update(override)

    credentials = Credentials.model_validate(obj=config)

    return credentials



from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class FavoriteName(BaseModel):
    name: str


class FavoriteId(BaseModel):
    data: int


class FavoritePosition(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    position: int
    list_id: int


class FavoriteItem(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    default: int | None = Field(default=None)
    id: int
    is_default: bool | None = Field(default=None)
    name: str | None = Field(default=None)


class FavoriteBatch(BaseModel):
    data: list[FavoriteItem] = Field(default_factory=list)



import logging

import requests
from orjson import loads

from degiro_connector.core.constants import urls
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.account import AccountUpdate, UpdateRequest
from degiro_connector.core.abstracts.abstract_action import AbstractAction


class ActionGetUpdate(AbstractAction):
    # ACTION_MATCHING = {
    #     "B": Order.Action.Value("BUY"),
    #     "S": Order.Action.Value("SELL"),
    # }
    # ORDER_MATCHING = {
    #     "buysell": "action",
    #     "contractSize": "contract_size",
    #     "contractType": "contract_type",
    #     "currency": "currency",
    #     "date": "hour",
    #     "id": "id",
    #     "isDeletable": "is_deletable",
    #     "isModifiable": "is_modifiable",
    #     "orderTimeTypeId": "time_type",
    #     "orderTypeId": "order_type",
    #     "price": "price",
    #     "product": "product",
    #     "productId": "product_id",
    #     "quantity": "quantity",
    #     "size": "size",
    #     "stopPrice": "stop_price",
    #     "totalOrderValue": "total_order_value",
    # }

    @staticmethod
    def build_model(response: requests.Response) -> AccountUpdate:
        model = AccountUpdate.model_validate_json(json_data=response.text)

        return model

    @staticmethod
    def build_params_map(request_list: list[UpdateRequest]) -> dict:
        params_map = {}

        for update_request in request_list:
            params_map[update_request.option.value] = update_request.last_updated

        return params_map

    @classmethod
    def get_update(
        cls,
        request_list: list[UpdateRequest],
        session_id: str,
        credentials: Credentials,
        logger: logging.Logger | None = None,
        raw: bool = False,
        session: requests.Session | None = None,
    ) -> AccountUpdate | dict | None:
        """Retrieve information from Degiro's Trading Update endpoint.
        Args:
            request_list (list[UpdateRequest]):
                list of options that we want to retrieve from the endpoint.
                Example :
                    request_list = [
                        UpdateRequest(
                            option=UpdateOption.ALERTS,
                            last_updated=0,
                        ),
                        UpdateRequest(
                            option=UpdateOption.CASH_FUNDS,
                            last_updated=0,
                        ),
                        UpdateRequest(
                            option=UpdateOption.HISTORICAL_ORDERS,
                            last_updated=0,
                        ),
                        UpdateRequest(
                            option=UpdateOption.ORDERS,
                            last_updated=0,
                        ),
                        UpdateRequest(
                            option=UpdateOption.PORTFOLIO,
                            last_updated=0,
                        ),
                        UpdateRequest(
                            option=UpdateOption.TOTAL_PORTFOLIO,
                            last_updated=0,
                        ),
                        UpdateRequest(
                            option=UpdateOption.TRANSACTIONS,
                            last_updated=0,
                        ),
                    ]
            session_id (str):
                API's session id.
            credentials (Credentials):
                Credentials containing the parameter "int_account".
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            Update: API response.
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.UPDATE
        url = f"{url}/{int_account};jsessionid={session_id}"
        params_map = cls.build_params_map(request_list=request_list)
        params_map.update({"intAccount": int_account, "sessionId": session_id})

        request = requests.Request(
            method="GET",
            params=params_map,
            url=url,
        )
        prepped = session.prepare_request(request)

        try:
            response = session.send(prepped)
            response.raise_for_status()

            if raw is True:
                model = loads(response.text)
            else:
                model = cls.build_model(response=response)
            return model
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        request_list: list[UpdateRequest],
        raw: bool = False,
    ) -> AccountUpdate | dict | None:
        credentials = self.credentials
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        logger = self.logger

        return self.get_update(
            request_list=request_list,
            session_id=session_id,
            credentials=credentials,
            logger=logger,
            raw=raw,
            session=session,
        )



import logging

import onetimepass as otp
import requests

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.login import Login, LoginError, LoginSuccess


class ActionConnect(AbstractAction):
    @classmethod
    def get_session_id(
        cls,
        credentials: Credentials,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> str:
        """Establish a connection with Degiro's Trading API.
        Args:
            credentials (Credentials):
                credentials.int_account (int)
                    Account unique identifer in Degiro's system.
                    It is optional.
                credentials.password (str)
                    Password used to log in the website.
                    It is mandatory.
                credentials.username (str)
                    Username used to log in the website.
                    It is mandatory.
                credentials.totp_secret is optional.
                    Secret code for Two-factor Authentication (2FA).
                    It is optional.
            session (requests.Session, optional):
                If you one wants to reuse existing "Session" object.
                Defaults to None.
            logger (logging.Logger, optional):
                If you one wants to reuse existing "Logger" object.
                Defaults to None.
        Raises:
            ConnectionError: Connection failed.
        Returns:
            str: Session id
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        if credentials.one_time_password or credentials.totp_secret_key:
            url = urls.LOGIN + "/totp"

            if credentials.one_time_password:
                one_time_password = str(credentials.one_time_password)
            else:
                totp_secret_key = credentials.totp_secret_key
                one_time_password = str(otp.get_totp(totp_secret_key))
        else:
            url = urls.LOGIN
            one_time_password = None

        login = Login(
            username=credentials.username,
            password=credentials.password,
            is_pass_code_reset=False,
            is_redirect_to_mobile=False,
            query_tarams={},
            one_time_password=one_time_password,
        )
        payload = login.model_dump(
            by_alias=True,
            exclude_none=True,
            mode="json",
        )
        request = requests.Request(
            method="POST",
            url=url,
            json=payload,
        )
        prepped = session.prepare_request(request)
        login_error = None
        login_sucess = None

        try:
            response = session.send(prepped)

            if response.status_code == 200:
                login_sucess = LoginSuccess.model_validate_json(json_data=response.text)
            else:
                login_error = LoginError.model_validate_json(json_data=response.text)
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
        except Exception as e:
            logger.fatal(e)

        if login_error:
            logger.fatal(
                "login_error:%s",
                login_error.model_dump(mode="python", by_alias=True, exclude_none=True),
            )

        if login_error and login_error.status == 6:
            raise ConnectionError('2FA is enabled, please provide the "totp_secret".')

        if login_sucess is None:
            raise ConnectionError("No session id returned.")

        logger.info(
            "login_sucess: %s",
            login_sucess.model_dump(mode="python", by_alias=True, exclude_none=True),
        )

        return login_sucess.session_id

    def call(self):
        connection_storage = self.connection_storage
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        connection_storage.session_id = self.get_session_id(
            credentials=credentials,
            logger=logger,
            session=session,
        )

        return connection_storage.session_id



from datetime import date, datetime, timedelta
from enum import Enum
from typing import Literal

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class Action(str, Enum):
    BUY = "BUY"
    SELL = "SELL"


class OrderType(int, Enum):
    LIMIT = 0
    MARKET = 2
    STOP_LIMIT = 1
    STOP_LOSS = 3

    # LIMIT_HIT = ?;
    # TRAILING_STOP = ?;
    # JOIN = ?;
    # STANDARD_SIZE = ?;
    # STANDARD_AMOUNT = ?;
    # TAKE_PROFIT = ?;
    # COMBINED = ?;
    # OCO = ?;


class TimeType(int, Enum):
    GOOD_TILL_CANCELED = 3
    GOOD_TILL_DAY = 1
    UNKNOWN_0 = 0
    UNKNOWN_2 = 2


class Order(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    # USED IN CHECKING
    buy_sell: Action | None = Field(default=None)
    id: str | None = Field(default=None)
    order_type: OrderType | None = Field(default=None)
    price: float | None = Field(default=None)
    product_id: int | None = Field(default=None)
    size: float | None = Field(default=None)
    stop_price: float = Field(default=None)
    time_type: TimeType = Field(default=None)

    # USED IN CONFIRMATION
    contract_size: float | None = Field(default=None)
    contract_type: int | None = Field(default=None)
    currency: str | None = Field(default=None)
    date: str | None = Field(
        default=None
    )  # hour : called "date" by the API contains %H:%M if the date was not passed
    is_deletable: bool | None = Field(default=None)
    is_modifiable: bool | None = Field(default=None)
    product: str | None = Field(default=None)
    quantity: float | None = Field(default=None)
    total_order_value: float | None = Field(default=None)

    # USED IN HISTORY
    retained_order: bool | None = Field(alias="retainedOrder", default=None)
    sent_to_exchange: bool | None = Field(alias="sentToExchange", default=None)


class CheckingResponse(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    auto_fx_conversion_rate: float | None = Field(default=None)
    confirmation_id: str
    free_space_new: float | None = Field(default=None)
    response_datetime: datetime | None = Field(default=None)
    request_duration: timedelta | None = Field(default=None)
    transaction_auto_fx_opposite_surcharges: list[dict] | None = Field(default=None)
    transaction_auto_fx_surcharges: list[dict] | None = Field(default=None)
    transaction_fee: float | None = Field(default=None)
    transaction_fees: list[dict] | None = Field(default=None)
    transaction_opposite_fees: list[dict] | None = Field(default=None)
    transaction_taxes: list[dict] | None = Field(default=None)
    show_ex_ante_report_link: bool | None = Field(default=None)


class CheckingWrapper(BaseModel):
    data: CheckingResponse


class ConfirmationResponse(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )
    order_id: str
    response_datetime: datetime | None = Field(default=None)
    request_duration: timedelta | None = Field(default=None)


class ConfirmationWrapper(BaseModel):
    data: ConfirmationResponse


ORDER_FIELD_MAP = {
    OrderType.LIMIT: {
        "buySell",
        "orderType",
        "price",
        "productId",
        "size",
        "timeType",
    },
    OrderType.STOP_LIMIT: {
        "buySell",
        "orderType",
        "price",
        "productId",
        "size",
        "stopPrice",
        "timeType",
    },
    OrderType.MARKET: {
        "buySell",
        "orderType",
        "productId",
        "size",
        "timeType",
    },
    OrderType.STOP_LOSS: {
        "buySell",
        "orderType",
        "productId",
        "size",
        "stopPrice",
        "timeType",
    },
}


class HistoryItem(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    buysell: Literal["B", "S"]
    created: datetime
    current_traded_size: int
    active: bool
    last: datetime
    order_id: str | None = Field(default=None)
    order_time_type_id: TimeType
    order_type_id: OrderType
    price: float
    product_id: int
    size: int
    status: str
    stop_price: float
    total_traded_size: int
    type: str


class History(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    data: list[HistoryItem] = Field(default_factory=list)


class HistoryRequest(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        json_encoders={date: lambda v: v.strftime("%d/%m/%Y") if v else None},
    )

    from_date: date
    to_date: date

    int_account: int | None = Field(default=None)
    session_id: str | None = Field(default=None)



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class ProductItem(BaseModel):
    model_config = ConfigDict(
        extra="allow",
        alias_generator=to_camel,
        populate_by_name=True,
    )

    active: bool | None = Field(default=None)
    buy_order_types: list[str] | None = Field(default=None)
    category: str | None = Field(default=None)
    close_price: float | None = Field(default=None)
    close_price_date: str | None = Field(default=None)
    contract_size: int | None = Field(default=None)
    currency: str | None = Field(default=None)
    exchange_id: str | None = Field(default=None)
    feed_quality: str | None = Field(default=None)
    feed_quality_secondary: str | None = Field(default=None)
    id: str | None = Field(default=None)
    is_shortable: bool | None = Field(default=None)
    isin: str | None = Field(default=None)
    name: str | None = Field(default=None)
    only_eod_prices: bool | None = Field(default=None)
    order_book_depth: int | None = Field(default=None)
    order_book_depth_secondary: int | None = Field(default=None)
    order_time_types: list[str] | None = Field(default=None)
    product_bit_types: list[str] | None = Field(default=None)
    product_type: str | None = Field(default=None)
    product_type_id: int | None = Field(default=None)
    quality_switch_free: bool | None = Field(default=None)
    quality_switch_free_secondary: bool | None = Field(default=None)
    quality_switchable: bool | None = Field(default=None)
    quality_switchable_secondary: bool | None = Field(default=None)
    sell_order_types: list[str] | None = Field(default=None)
    strike_price: float | None = Field(default=None)
    symbol: str | None = Field(default=None)
    tradable: bool | None = Field(default=None)
    vwd_id: str | None = Field(default=None)
    vwd_id_secondary: str | None = Field(default=None)
    vwd_identifier_type: str | None = Field(default=None)
    vwd_identifier_type_secondary: str | None = Field(default=None)
    vwd_module_id: int | None = Field(default=None)
    vwd_module_id_secondary: int | None = Field(default=None)


class ProductInfo(BaseModel):
    data: dict[int, ProductItem] = Field(default_factory=dict)


class FinancialStatements(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        extra="allow",
        populate_by_name=True,
    )

    annual: list[dict] = Field(default_factory=list)
    currency: str | None = Field(default=None)
    interim: list[dict] = Field(default_factory=list)


class StatementsWrapper(BaseModel):
    data: FinancialStatements


class EstimatesSummaries(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        extra="allow",
        populate_by_name=True,
    )
    annual: list[dict] = Field(default_factory=list)
    interim: list[dict] = Field(default_factory=list)
    ric: str | None = Field(default=None)
    currency: str | None = Field(default=None)
    lastUpdated: datetime | None = Field(default=None)
    lastRetrieved: datetime | None = Field(default=None)
    preferredMeasure: str | None = Field(default=None)


class SummariesWrapper(BaseModel):
    data: EstimatesSummaries



import logging
import time
from datetime import datetime, timedelta
from typing import Any

import requests
from orjson import loads

from degiro_connector.core.constants import urls
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.order import (
    CheckingResponse,
    CheckingWrapper,
    Order,
    ORDER_FIELD_MAP,
)
from degiro_connector.core.abstracts.abstract_action import AbstractAction


class ActionCheckOrder(AbstractAction):
    @staticmethod
    def build_json_map(order: Order) -> dict:
        json_map = order.model_dump(
            by_alias=True,
            exclude_none=True,
            mode="json",
        )

        if order.order_type not in ORDER_FIELD_MAP:
            raise AttributeError("Invalid `OrderType`.")

        if order.buy_sell is None:
            raise AttributeError("Invalid `buy_sell`.")

        field_list = ORDER_FIELD_MAP[order.order_type]
        json_map = {field: json_map[field] for field in field_list if field in json_map}

        return json_map

    @staticmethod
    def build_model(
        response: requests.Response,
        duration_ns: int,
    ) -> CheckingResponse:
        model = CheckingWrapper.model_validate_json(
            json_data=response.text,
        ).data
        model.response_datetime = datetime.now()
        model.request_duration = timedelta(microseconds=duration_ns // 1000)

        return model

    @classmethod
    def check_order(
        cls,
        credentials: Credentials,
        order: Order,
        session_id: str,
        logger: logging.Logger | None = None,
        raw: bool = False,
        session: requests.Session | None = None,
    ) -> CheckingResponse | Any | None:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.ORDER_CHECK
        url = f"{url};jsessionid={session_id}"
        params = {"intAccount": int_account, "sessionId": session_id}
        json_map = cls.build_json_map(order=order)
        request = requests.Request(
            method="POST",
            url=url,
            json=json_map,
            params=params,
        )
        prepped = session.prepare_request(request)
        start_ns = time.perf_counter_ns()

        try:
            response = session.send(prepped)
            duration_ns = time.perf_counter_ns() - start_ns
            response.raise_for_status()

            if raw is True:
                model = loads(response.text)
            else:
                model = cls.build_model(
                    response=response,
                    duration_ns=duration_ns,
                )
            return model
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        order: Order,
        raw: bool = False,
    ) -> CheckingResponse | Any | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        credentials = self.credentials
        session = self.session_storage.session
        logger = self.logger

        return self.check_order(
            credentials=credentials,
            order=order,
            session_id=session_id,
            logger=logger,
            raw=raw,
            session=session,
        )



from datetime import datetime
from enum import Enum

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class CalendarType(str, Enum):
    DIVIDEND_CALENDAR = "DividendCalendar"
    ECONOMIC_CALENDAR = "EconomicCalendar"
    EARNINGS_CALENDAR = "EarningsCalendar"
    HOLIDAY_CALENDAR = "HolidayCalendar"
    IPO_CALENDAR = "IpoCalendar"
    SPLIT_CALENDAR = "SplitCalendar"


class AgendaRequest(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        extra="allow",
        populate_by_name=True,
        json_encoders={
            datetime: lambda v: v.strftime("%Y-%m-%dT%H:%M:%SZ") if v else None
        },
    )

    calendar_type: CalendarType
    classifications: str | None = Field(default=None)
    company_name: str | None = Field(default=None)
    countries: str | None = Field(default=None)
    end_date: datetime
    limit: int = Field(default=25)
    offset: int = Field(default=0)
    sort_column: str | None = Field(default="date")
    sort_type: str | None = Field(default="asc")
    start_date: datetime
    units: str | None = Field(default=None)

    int_account: int | None = Field(default=None)
    session_id: str | None = Field(default=None)


class Agenda(BaseModel):
    items: list[dict]
    offset: int
    total: int



from datetime import date, datetime

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class HistoryRequest(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        json_encoders={date: lambda v: v.strftime("%d/%m/%Y") if v else None},
    )

    from_date: date
    to_date: date
    group_transactions_by_order: bool = Field(default=False)

    int_account: int | None = Field(default=None)
    session_id: str | None = Field(default=None)


class HistoryItem(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        extra="allow",
        populate_by_name=True,
    )

    auto_fx_fee_in_base_currency: float | None = Field(default=None)
    buysell: str | None = Field(default=None)
    counter_party: str | None = Field(default=None)
    date: datetime | None = Field(default=None)
    executing_entity_id: str | None = Field(default=None)
    fee_in_base_currency: float | None = Field(default=None)
    fx_rate: float | None = Field(default=None)
    gross_fx_rate: float | None = Field(default=None)
    id: int | None = Field(default=None)
    nett_fx_rate: float | None = Field(default=None)
    order_type_id: int | None = Field(default=None)
    price: float | None = Field(default=None)
    product_id: int | None = Field(default=None)
    quantity: int | None = Field(default=None)
    total: float | None = Field(default=None)
    total_fees_in_base_currency: float | None = Field(default=None)
    total_in_base_currency: float | None = Field(default=None)
    total_plus_all_fees_in_base_currency: float | None = Field(default=None)
    total_plus_fee_in_base_currency: float | None = Field(default=None)
    transfered: bool | None = Field(default=None)
    trading_venue: str | None = Field(default=None)
    transaction_type_id: int | None = Field(default=None)


class TransactionsHistory(BaseModel):
    data: list[HistoryItem] = Field(default_factory=list)



import orjson as json


from degiro_connector.quotecast.models.message import (
    Message,
    MessageNumeric,
    MessageRegistration,
    MessageText,
    MessageUnregistration,
)
from degiro_connector.quotecast.models.metric import (
    Metric,
    MetricType,
)
from degiro_connector.quotecast.models.ticker import Ticker


class TickerToMetricList:
    """Handle the payload returned from this endpoint :

    "https://degiro.quotecast.vwdservices.com/CORS/{session_id}"

    OVERALL EXPLANATION
    The endpoint aforementioned returns financial data measurement
    such as :
        * LastPrice of a Stock
        * LastVolume of a Stock
        * ...

    This endpoint use HTTP 1.1 protocol.

    Since there are no way to retrieve data asynchronously from
    HTTP 1.1, this endpoint uses Long-Polling method to retrieve data
    as soon as they are available.

    Thus to consume this endpoint one needs to recall it as soon as a
    response has been received.

    If no data updates are available a HEARTBEAT signal is sent every
    5 seconds.

    The session timeout is approximativaly 15 seconds.

    PAYLOAD DATA DESCRIPTION
    The payload sent from this endpoint can contain three kind of
    objects :
        * DATA
        * HEARTBEAT
        * MATCHING

    A DATA is an object with two elements :
        * REFERENCE
        * VALUE

    A HEARTBEAT is signal sent every 5 seconds if no data updates
    are available.

    A MATCHING is an object with three elements :
        * REFERENCE
        * VWD_ID
        * LABEL

    The REFERENCE is a unique identifier in Degiro's Quotecast API
    which refers to the financial data.

    The VALUE is the value of the financial data.

    THE VWD_ID is the id of the product (etf, option, stock, warrant...)
    from which we retrieve measurements (LastPrice, LastVolume...).

    The LABEL is the name of the measurements that we retrieve.

    The MATCHING table is only sent at the first call of this endpoint.

    PAYLOAD DATA STRUCTURE
    The payload is a list of messages serialized in JSON, as follows :
    ```json
    [
        MESSAGE_1,
        MESSAGE_2,
        ...
        MESSAGE_X
    ]
    ```

    What is called a MESSAGE here above, is an object with the two
    attributes :
        * "m"
        * "v"

    We can denote a message like this :
    ```python
    MESSAGE = {
        'm': MESSAGE_TYPE
        'v': [CODE1, CODE2]
    }
    ```

    Here we have the following properties:
        * m contains a MESSAGE_TYPE
        * MESSAGE_TYPE is a str
        * v is a list
        * v contains 2 elements
        * CODE1 is a str or an int
        * CODE2 is a float

    The MESSAGE_TYPE indicate the type of message, it can take the
    following __values :
        * "a_req" : subscription
        * "a_rel" : unsubscription
        * "d" : rejected subscription
        * "h" : heartbeat
        * "sr" : session invalid
        * "ue" : data not available
        * "un" : numeric data
        * "us" : string data
        * ... (this list may not be exhaustive)

    Depending on the MESSAGE_TYPE different kind of information are stored
    inside :
        * CODE1
        * CODE2

    If MESSAGE_TYPE = "a_req" or "a_rel" :
        * CODE1 : contains the product's VWD_ID and the PARAMETER_NAME.
        * CODE2 : contains the REFERENCE for the parameter in CODE1.
        * Example of MESSAGE :
        ```json
        {
            "m": "a_req",
            "v": ["365004197.B10Volume", 624239]
        }
        ```
        "365004197.B10Volume" <=> CODE1
        365004197 <=> VWD_ID
        B10Volume <=> PARAMETER_NAME
        624239 <=> REFERENCE

    If MESSAGE_TYPE = "un" or "us :
        * CODE1 : contains the reference number.
        * CODE2 : contains the value of the information referenced by
        CODE1.
        * Example of MESSAGE :
        ```json
        {
            "m": "a_req",
            "v": [624239, 115.85]
        }
        ```
        624239 <=> CODE1
        115.85 <=> CODE2
    """

    @staticmethod
    def from_ticker_to_message_list(ticker: Ticker) -> list[Message]:
        json_text = ticker.json_text
        message_list_raw = json.loads(json_text)  # pylint: disable=no-member
        message_list: list[Message] = []

        for message_raw in message_list_raw:
            if message_raw["m"] == "un":
                message_list.append(
                    MessageNumeric(
                        reference=message_raw["v"][0],
                        value=message_raw["v"][1],
                    ),
                )
            elif message_raw["m"] == "us":
                message_list.append(
                    MessageText(
                        reference=message_raw["v"][0],
                        value=message_raw["v"][1],
                    ),
                )
            elif message_raw["m"] == "a_req":
                message_list.append(
                    MessageRegistration(
                        metric_name=message_raw["v"][0],
                        reference=message_raw["v"][1],
                    ),
                )
            elif message_raw["m"] == "a_rel":
                message_list.append(
                    MessageUnregistration(
                        metric_name=message_raw["v"][0],
                        reference=message_raw["v"][1],
                    ),
                )
            elif message_raw["m"] == "h":
                pass
            elif message_raw["m"] == "ue":
                pass
            elif message_raw["m"] == "d":
                raise AttributeError(
                    f"Subscription rejected, the `vwd_id` or `metric` might not exist. - {message_raw}"
                )
            else:
                raise AttributeError(f"Unknown metric : {message_raw}")

        return message_list

    def __init__(
        self,
        reference_map: dict[int, list] | None = None,
    ) -> None:
        """

        Parameters
        ----------
            reference_map: dict[int, list] | None
                Dictionnary storing the references returned by Degiro's Quotecast.
                Each reference number matches with a specific product/metric_type set.
                Example : {reference_number: [product_id, metric_type]}
        """
        # {reference: [product_id, metric_type]}
        self._reference_map: dict[int, list] = reference_map or {}

    def from_message_list_to_metric_list(
        self, message_list: list[Message]
    ) -> list[Metric]:
        reference_map = self._reference_map
        metric_list = []

        for message in message_list:
            if isinstance(message, MessageRegistration):
                reference_map[message.reference] = message.metric_name.rsplit(
                    sep=".", maxsplit=1
                )
            elif isinstance(message, MessageUnregistration):
                del reference_map[
                    message.reference
                ]  # crashes on purpose to detect inconsistency
            elif isinstance(message, (MessageNumeric, MessageText)):
                product_id, metric_type = reference_map[message.reference]
                metric_list.append(
                    Metric(
                        product_id=product_id,
                        metric_type=MetricType(metric_type),
                        value=message.value,
                    ),
                )

        return metric_list

    def parse(self, ticker: Ticker) -> list[Metric]:
        message_list = self.from_ticker_to_message_list(ticker=ticker)
        metric_list = self.from_message_list_to_metric_list(message_list=message_list)
        return metric_list



import logging


import requests
from orjson import loads

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.news import (
    LatestNews,
    LatestRequest,
    LatestWrapper,
)


class ActionGetLatestNews(AbstractAction):
    @staticmethod
    def build_model(response: requests.Response) -> LatestNews:
        model = LatestWrapper.model_validate_json(json_data=response.text).data

        return model

    @staticmethod
    def build_params_map(
        latest_request: LatestRequest,
    ) -> dict:
        params_map = latest_request.model_dump(
            by_alias=True,
            exclude_none=True,
            mode="json",
        )

        return params_map

    @classmethod
    def get_latest_news(
        cls,
        latest_request: LatestRequest,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> LatestNews | dict | None:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.LATEST_NEWS

        params_map = cls.build_params_map(latest_request=latest_request)
        params_map.update({"intAccount": int_account, "sessionId": session_id})

        request = requests.Request(
            method="GET",
            params=params_map,
            url=url,
        )
        prepped = session.prepare_request(request)
        prepped.headers["cookie"] = "JSESSIONID=" + session_id

        try:
            response = session.send(prepped)
            response.raise_for_status()

            if raw is True:
                model = loads(response.text)
            else:
                model = cls.build_model(response=response)
            return model
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        latest_request: LatestRequest,
        raw: bool = False,
    ) -> LatestNews | dict | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.get_latest_news(
            latest_request=latest_request,
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )



import abc
import logging
import requests
from inspect import ismethod


from degiro_connector.core.models.model_connection import ModelConnection
from degiro_connector.core.models.model_session import ModelSession


class AbstractAction(abc.ABC):
    # COMMENTING @final : FOR COMPATIBILITY WITH PYTHON 3.7
    # @final
    @staticmethod
    def build_logger() -> logging.Logger:
        return logging.getLogger(__name__)

    # @final
    @staticmethod
    def build_session(headers: dict[str, str] | None = None) -> requests.Session:
        return ModelSession.build_session()

    # @final
    @property
    def credentials(self):
        return self._credentials

    # @final
    @property
    def connection_storage(self):
        return self._connection_storage

    # @final
    @property
    def logger(self):
        return self._logger

    # @final
    @property
    def session_storage(self):
        return self._session_storage

    # @final
    def __init__(
        self,
        credentials,
        connection_storage: ModelConnection,
        logger: logging.Logger | None = None,
        session_storage: ModelSession = None,
        *args,
        **kwargs,
    ):
        # Each action implement a `call` method.
        # This `call` method can have it's own set of *args and **kwargs
        assert hasattr(self, "call") and ismethod(getattr(self, "call"))

        self._credentials = credentials
        self._connection_storage = connection_storage
        self._logger = logger or logging.getLogger(self.__module__)
        self._session_storage = session_storage or ModelSession(
            hooks=self._connection_storage.build_hooks(),
            ssl_check=True,
        )

        self.post_init(*args, **kwargs)

    # @final
    def __call__(self, *args, **kwargs):
        return self.call(*args, **kwargs)

    def post_init(self, *args, **kwargs):
        pass



import logging

import requests

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials


class ActionDeleteOrder(AbstractAction):
    @classmethod
    def delete_order(
        cls,
        order_id: str,
        session_id: str,
        credentials: Credentials,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> bool | None:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.ORDER_DELETE
        url = f"{url}/{order_id};jsessionid={session_id}"
        params = {"intAccount": int_account, "sessionId": session_id}

        request = requests.Request(method="DELETE", url=url, params=params)
        prepped = session.prepare_request(request)

        try:
            response = session.send(prepped)
            response.raise_for_status()

            return response.status_code == 200
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        order_id: str,
    ) -> bool | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.delete_order(
            order_id=order_id,
            session_id=session_id,
            credentials=credentials,
            session=session,
            logger=logger,
        )



from datetime import datetime, date
from enum import Enum

from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel


class OverviewRequest(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        json_encoders={date: lambda v: v.strftime("%d/%m/%Y") if v else None},
    )

    from_date: date
    to_date: date


class CashMovements(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )

    balance: dict | None = Field(default=None)
    change: float | None = Field(default=None)
    currency: str | None = Field(default=None)
    date: datetime | None = Field(default=None)
    description: str | None = Field(default=None)
    id: int | None = Field(default=None)
    product_id: int | None = Field(default=None)
    type: str | None = Field(default=None)
    value_date: datetime | None = Field(default=None)


class AccountOverview(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )
    cash_movements: list[CashMovements] | None = Field(default=None)


class OverviewWrapper(BaseModel):
    data: AccountOverview


class Format(str, Enum):
    CSV = "csv"
    HTML = "html"
    PDF = "pdf"
    XLS = "xls"


class ReportRequest(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        json_encoders={date: lambda v: v.strftime("%d/%m/%Y") if v else None},
    )

    country: str
    lang: str
    format: Format = Field(default=Format.CSV)
    from_date: date
    to_date: date

    int_account: int | None = Field(default=None)
    session_id: str | None = Field(default=None)


class Report(BaseModel):
    content: str
    format: Format


class UpdateOption(str, Enum):
    ALERTS = "alerts"
    CASH_FUNDS = "cashFunds"
    HISTORICAL_ORDERS = "historicalOrders"
    ORDERS = "orders"
    PORTFOLIO = "portfolio"
    TOTAL_PORTFOLIO = "totalPortfolio"
    TRANSACTIONS = "transactions"


class UpdateRequest(BaseModel):
    option: UpdateOption
    last_updated: int = Field(default=0)


class AccountUpdate(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        extra="allow",
        populate_by_name=True,
    )

    alerts: dict | None = Field(default=None)
    cash_funds: dict | None = Field(default=None)
    historical_orders: dict | None = Field(default=None)
    orders: dict | None = Field(default=None)
    portfolio: dict | None = Field(default=None)
    total_portfolio: dict | None = Field(default=None)
    transactions: dict | None = Field(default=None)



import logging

import requests

from degiro_connector.core.constants import urls
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.core.abstracts.abstract_action import AbstractAction


class ActionLogout(AbstractAction):
    @classmethod
    def logout(
        cls,
        credentials: Credentials,
        session_id: str,
        logger: logging.Logger | None = None,
        session: requests.Session | None = None,
    ) -> bool | None:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.LOGOUT
        url = f"{url};jsessionid={session_id}"

        params = {
            "intAccount": int_account,
            "sessionId": session_id,
        }

        request = requests.Request(
            method="PUT",
            url=url,
            params=params,
        )
        prepped = session.prepare_request(request)
        response_raw = None

        try:
            response_raw = session.send(prepped)
            response_raw.raise_for_status()
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            status_code = getattr(response_raw, "status_code", "No status_code found.")
            text = getattr(response_raw, "text", "No text found.")
            logger.fatal(status_code)
            logger.fatal(text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

        return response_raw.status_code == 200

    def call(self) -> bool | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        logger = self.logger
        credentials = self.credentials

        return self.logout(
            credentials=credentials,
            session_id=session_id,
            logger=logger,
            session=session,
        )



import logging

import requests

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.order import Order, ORDER_FIELD_MAP


class ActionUpdateOrder(AbstractAction):
    @staticmethod
    def build_json_map(order: Order) -> dict:
        json_map = order.model_dump(
            by_alias=True,
            exclude_none=True,
            mode="json",
        )

        if order.id is None:
            raise AttributeError("No `order.id` provided.")

        if order.order_type not in ORDER_FIELD_MAP:
            raise AttributeError("Invalid `OrderType`.")

        if order.buy_sell is None:
            raise AttributeError("Invalid `buy_sell`.")

        field_list = ORDER_FIELD_MAP[order.order_type]
        json_map = {field: json_map[field] for field in field_list if field in json_map}

        return json_map

    @classmethod
    def update_order(
        cls,
        order: Order,
        session_id: str,
        credentials: Credentials,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> bool | None:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        order_id = order.id
        url = urls.ORDER_UPDATE
        url = f"{url}/{order_id};jsessionid={session_id}"
        params = {"intAccount": int_account, "sessionId": session_id}
        json_map = cls.build_json_map(order=order)
        request = requests.Request(
            method="PUT",
            url=url,
            json=json_map,
            params=params,
        )
        prepped = session.prepare_request(request)
        response_raw = None

        try:
            response_raw = session.send(prepped)
            response_raw.raise_for_status()

            return response_raw.status_code == 200
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        order: Order,
    ) -> bool | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.update_order(
            order=order,
            session_id=session_id,
            credentials=credentials,
            session=session,
            logger=logger,
        )



import logging
import threading

import requests

import degiro_connector.core.constants.headers as default_headers


class ModelSession:
    """Handle the Requests Session objects in a threadsafe manner."""

    @staticmethod
    def build_session(
        headers: dict | None = None,
        hooks: dict | None = None,
    ) -> requests.Session:
        """Setup a "requests.Session" object.
        Args:
            headers (dict, optional):
                Headers to used for the Session.
                Defaults to None.
            hooks (dict, optional):
                Hooks for the Session.
                Defaults to None.

        Returns:
            requests.Session:
                Session object with the right headers and hooks.
        """

        session = requests.Session()

        if isinstance(headers, dict):
            session.headers.update(headers)
        else:
            session.headers.update(default_headers.HEADERS)

        if isinstance(hooks, dict):
            session.hooks.update(hooks)

        return session

    @property
    def session(self) -> requests.Session:
        self.__logger.debug("session:getter: %s", threading.current_thread().name)

        if not hasattr(self.__local_storage, "session"):
            self.__local_storage.session = self.build_session(
                headers=self.__headers,
                hooks=self.__hooks,
            )

        return self.__local_storage.session

    @session.setter
    def session(self, session: requests.Session):
        self.__logger.debug("session:setter: %s", threading.current_thread().name)

        self.__local_storage.session = session

    def reset_session(
        self,
        headers: dict | None = None,
        hooks: dict | None = None,
    ):
        self.__local_storage.session = self.build_session(
            headers=headers,
            hooks=hooks,
        )

    def __init__(
        self,
        headers: dict | None = None,
        hooks: dict | None = None,
    ):
        self.__logger = logging.getLogger(self.__module__)
        self.__local_storage = threading.local()

        if isinstance(headers, dict):
            headers = dict(headers)

        if isinstance(hooks, dict):
            hooks = dict(hooks)

        self.__headers = headers
        self.__hooks = hooks



import logging

import requests

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.favorite import FavoritePosition


class ActionMoveFavorite(AbstractAction):
    @classmethod
    def move_favorite(
        cls,
        list_id: int,
        position: int,
        session_id: str,
        credentials: Credentials,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> bool | None:
        """Move a favorite list.
        Args:
            list_id (int):
                Id of the list.
            position (int):
                Expected position.
            session_id (str):
                API's session id.
            credentials (Credentials):
                Credentials containing the parameter "int_account".
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            Favorites: API response.
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = f"{urls.FAVOURITES_LIST}/reorder"

        params = {
            "intAccount": int_account,
            "sessionId": session_id,
        }

        json_obj = FavoritePosition(list_id=list_id, position=position).model_dump(
            by_alias=True,
            exclude_none=True,
            mode="json",
        )

        request = requests.Request(
            method="PUT",
            url=url,
            params=params,
            json=json_obj,
        )
        prepped = session.prepare_request(request)

        try:
            response = session.send(prepped)
            response.raise_for_status()
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

        return response.status_code == 200

    def call(
        self,
        list_id: int,
        position: int,
    ) -> bool | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.move_favorite(
            list_id=list_id,
            position=position,
            session_id=session_id,
            credentials=credentials,
            session=session,
            logger=logger,
        )



import logging

import requests
from orjson import loads

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.product import EstimatesSummaries, SummariesWrapper


class ActionGetEstimatesSummaries(AbstractAction):
    @staticmethod
    def build_model(response: requests.Response) -> EstimatesSummaries:
        model = SummariesWrapper.model_validate_json(json_data=response.text).data

        return model

    @classmethod
    def get_estimates_summaries(
        cls,
        product_isin: str,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> EstimatesSummaries | dict | None:
        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = f"{urls.ESTIMATES_SUMMARIES}/{product_isin}"
        params_map = {"intAccount": int_account, "sessionId": session_id}

        request = requests.Request(
            method="GET",
            params=params_map,
            url=url,
        )
        prepped = session.prepare_request(request)
        prepped.headers["cookie"] = "JSESSIONID=" + session_id

        try:
            response = session.send(prepped)
            response.raise_for_status()

            if raw is True:
                model = loads(response.text)
            else:
                model = cls.build_model(response=response)
            return model
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        product_isin: str,
        raw: bool = False,
    ) -> EstimatesSummaries | dict | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.get_estimates_summaries(
            product_isin=product_isin,
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )



import logging

import requests
from orjson import loads

from degiro_connector.core.constants import urls
from degiro_connector.core.abstracts.abstract_action import AbstractAction
from degiro_connector.trading.models.credentials import Credentials
from degiro_connector.trading.models.order import HistoryRequest, History


class ActionGetOrdersHistory(AbstractAction):
    @staticmethod
    def build_model(response: requests.Response) -> History:
        model = History.model_validate_json(json_data=response.text)

        return model

    @staticmethod
    def build_params_map(history_request: HistoryRequest) -> dict:
        params_map = history_request.model_dump(
            by_alias=True,
            exclude_none=True,
            mode="json",
        )

        return params_map

    @classmethod
    def get_orders_history(
        cls,
        history_request: HistoryRequest,
        session_id: str,
        credentials: Credentials,
        raw: bool = False,
        session: requests.Session | None = None,
        logger: logging.Logger | None = None,
    ) -> History | dict | None:
        """Retrieve history about orders.
        Args:
            history_request (HistoryRequest):
                list of options that we want to retrieve from the endpoint.
                Example :
                    history_request = HistoryRequest(
                        from_date=date(year=date.today().year, month=1, day=1),
                        to_date=date.today(),
                    )
            session_id (str):
                API's session id.
            credentials (Credentials):
                Credentials containing the parameter "int_account".
            raw (bool, optional):
                Whether are not we want the raw API response.
                Defaults to False.
            session (requests.Session, optional):
                This object will be generated if None.
                Defaults to None.
            logger (logging.Logger, optional):
                This object will be generated if None.
                Defaults to None.
        Returns:
            OrdersHistory: API response.
        """

        if logger is None:
            logger = cls.build_logger()
        if session is None:
            session = cls.build_session()

        int_account = credentials.int_account
        url = urls.ORDERS_HISTORY
        params_map = cls.build_params_map(history_request=history_request)
        params_map.update({"intAccount": int_account, "sessionId": session_id})

        request = requests.Request(method="GET", url=url, params=params_map)
        prepped = session.prepare_request(request=request)

        try:
            response = session.send(prepped)
            response.raise_for_status()

            if raw is True:
                model = loads(response.text)
            else:
                model = cls.build_model(response=response)
            return model
        except requests.HTTPError as e:
            logger.fatal(e)
            if isinstance(e.response, requests.Response):
                logger.fatal(e.response.text)
            return None
        except Exception as e:
            logger.fatal(e)
            return None

    def call(
        self,
        history_request: HistoryRequest,
        raw: bool = False,
    ) -> History | dict | None:
        connection_storage = self.connection_storage
        session_id = connection_storage.session_id
        session = self.session_storage.session
        credentials = self.credentials
        logger = self.logger

        return self.get_orders_history(
            history_request=history_request,
            session_id=session_id,
            credentials=credentials,
            raw=raw,
            session=session,
            logger=logger,
        )
